@@ -32,63 +32,147 @@
 #include <errno.h>
 
 #include "gdk.h"
+#include "gdkx.h"
 
 #include "gdkprivate-x11.h"
 #include "gdkinternals.h"
+#include "gdkdisplay-x11.h"
 #include "gdkkeysyms.h"
 
 #include "config.h"
 
-guint _gdk_keymap_serial = 0;
-
-static gint min_keycode = 0;
-static gint max_keycode = 0;
-
-static inline void
-update_keyrange (void)
-{
-  if (max_keycode == 0)
-    XDisplayKeycodes (gdk_display, &min_keycode, &max_keycode);
-}
-
 #ifdef HAVE_XKB
 #include <X11/XKBlib.h>
 
 /* OSF-4.0 is apparently missing this macro
  */
-#ifndef XkbKeySymEntry
-#define	XkbKeySymEntry(d,k,sl,g) \
-	(XkbKeySym(d,k,((XkbKeyGroupsWidth(d,k)*(g))+(sl))))
+#  ifndef XkbKeySymEntry
+#    define XkbKeySymEntry(d,k,sl,g) \
+ 	(XkbKeySym(d,k,((XkbKeyGroupsWidth(d,k)*(g))+(sl))))
+#  endif
+#endif /* HAVE_XKB */
+
+typedef struct _GdkKeymapX11 GdkKeymapX11;
+
+#define GDK_TYPE_KEYMAP_X11          (gdk_keymap_x11_get_type ())
+#define GDK_KEYMAP_X11(object)       (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_KEYMAP_X11, GdkKeymapX11))
+#define GDK_IS_KEYMAP_X11(object)    (G_TYPE_CHECK_INSTANCE_TYPE ((object), GDK_TYPE_KEYMAP_X11))
+
+struct _GdkKeymapX11
+{
+  GdkKeymap     parent_instance;
+  
+  gint min_keycode;
+  gint max_keycode;
+  KeySym* keymap;
+  gint keysyms_per_keycode;
+  XModifierKeymap* mod_keymap;
+  guint lock_keysym;
+  GdkModifierType group_switch_mask;
+  GdkModifierType num_lock_mask;
+  gboolean sun_keypad;
+  PangoDirection current_direction;
+  gboolean have_direction;
+  guint current_serial;
+  
+#ifdef HAVE_XKB
+  XkbDescPtr xkb_desc;
+#endif
+};
+
+#define KEYMAP_USE_XKB(keymap) GDK_DISPLAY_X11 ((keymap)->display)->use_xkb
+#define KEYMAP_XDISPLAY(keymap) GDK_DISPLAY_XDISPLAY ((keymap)->display)
+
+static GType gdk_keymap_x11_get_type (void);
+static void  gdk_keymap_x11_init     (GdkKeymapX11 *keymap);
+
+static GType
+gdk_keymap_x11_get_type (void)
+{
+  static GType object_type = 0;
+
+  if (!object_type)
+    {
+      static const GTypeInfo object_info =
+	{
+	  sizeof (GdkKeymapClass),
+	  (GBaseInitFunc) NULL,
+	  (GBaseFinalizeFunc) NULL,
+	  (GClassInitFunc) NULL,
+	  NULL,           /* class_finalize */
+	  NULL,           /* class_data */
+	  sizeof (GdkKeymapX11),
+	  0,              /* n_preallocs */
+	  (GInstanceInitFunc) gdk_keymap_x11_init,
+	};
+      
+      object_type = g_type_register_static (GDK_TYPE_KEYMAP,
+                                            "GdkKeymapX11",
+                                            &object_info, 0);
+    }
+  
+  return object_type;
+}
+
+static void
+gdk_keymap_x11_init (GdkKeymapX11 *keymap)
+{
+  keymap->min_keycode = 0;
+  keymap->max_keycode = 0;
+
+  keymap->keymap = NULL;
+  keymap->keysyms_per_keycode = 0;
+  keymap->mod_keymap = NULL;
+  
+  keymap->num_lock_mask = 0;
+  keymap->sun_keypad = FALSE;
+  keymap->group_switch_mask = 0;
+  keymap->lock_keysym = GDK_Caps_Lock;
+  keymap->have_direction = FALSE;
+  
+#ifdef HAVE_XKB
+  keymap->xkb_desc = NULL;
 #endif
 
-gboolean _gdk_use_xkb = FALSE;
-gint _gdk_xkb_event_type;
-static XkbDescPtr xkb_desc = NULL;
+  keymap->current_serial = 0;
+}
+
+static inline void
+update_keyrange (GdkKeymapX11 *keymap_x11)
+{
+  if (keymap_x11->max_keycode == 0)
+    XDisplayKeycodes (KEYMAP_XDISPLAY (GDK_KEYMAP (keymap_x11)),
+		      &keymap_x11->min_keycode, &keymap_x11->max_keycode);
+}
+
+#ifdef HAVE_XKB
 
 static XkbDescPtr
-get_xkb (void)
+get_xkb (GdkKeymapX11 *keymap_x11)
 {
-  static guint current_serial = 0;
-
-  update_keyrange ();
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (GDK_KEYMAP (keymap_x11)->display);
+  Display *xdisplay = display_x11->xdisplay;
   
-  if (xkb_desc == NULL)
+  update_keyrange (keymap_x11);
+  
+  if (keymap_x11->xkb_desc == NULL)
     {
-      xkb_desc = XkbGetMap (gdk_display, XkbKeySymsMask | XkbKeyTypesMask, XkbUseCoreKbd);
-      if (xkb_desc == NULL)
-        g_error ("Failed to get keymap");
+      keymap_x11->xkb_desc = XkbGetMap (xdisplay, XkbKeySymsMask | XkbKeyTypesMask, XkbUseCoreKbd);
+      if (keymap_x11->xkb_desc == NULL)
+	g_error ("Failed to get keymap");
 
-      XkbGetNames (gdk_display, XkbGroupNamesMask, xkb_desc);
+      XkbGetNames (xdisplay, XkbGroupNamesMask, keymap_x11->xkb_desc);
     }
-  else if (current_serial != _gdk_keymap_serial)
+  else if (keymap_x11->current_serial != display_x11->keymap_serial)
     {
-      XkbGetUpdatedMap (gdk_display, XkbKeySymsMask | XkbKeyTypesMask, xkb_desc);
-      XkbGetNames (gdk_display, XkbGroupNamesMask, xkb_desc);
+      XkbGetUpdatedMap (xdisplay, XkbKeySymsMask | XkbKeyTypesMask,
+			keymap_x11->xkb_desc);
+      XkbGetNames (xdisplay, XkbGroupNamesMask, keymap_x11->xkb_desc);
     }
 
-  current_serial = _gdk_keymap_serial;
+  keymap_x11->current_serial = display_x11->keymap_serial;
 
-  return xkb_desc;
+  return keymap_x11->xkb_desc;
 }
 #endif /* HAVE_XKB */
 
@@ -96,66 +180,68 @@ get_xkb (void)
  * XkbSetDetectableAutorepeat. If FALSE, we'll fall back
  * to checking the next event with XPending().
  */
-gboolean _gdk_have_xkb_autorepeat = FALSE;
 
-static KeySym* keymap = NULL;
-static gint keysyms_per_keycode = 0;
-static XModifierKeymap* mod_keymap = NULL;
-static GdkModifierType group_switch_mask = 0;
-static PangoDirection current_direction;
-static gboolean       have_direction = FALSE;
-static GdkKeymap *default_keymap = NULL;
-
-/**
- * gdk_keymap_get_default:
- * 
- * Gets the #GdkKeymap for the default display.
- * 
- * Return value: the default keymap
+/** 
+ * gdk_keymap_get_for_display:
+ * @display: the #GdkDisplay.
+ * @returns: the #GdkKeymap attached to @display.
+ *
+ * Returns the #GdkKeymap attached to @display.
+ *
+ * Since: 2.2
  **/
 GdkKeymap*
-gdk_keymap_get_default (void)
+gdk_keymap_get_for_display (GdkDisplay *display)
 {
-  if (default_keymap == NULL)
-    default_keymap = g_object_new (gdk_keymap_get_type (), NULL);
+  GdkDisplayX11 *display_x11;
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+  display_x11 = GDK_DISPLAY_X11 (display);
+  
+  if (!display_x11->keymap)
+    display_x11->keymap = g_object_new (gdk_keymap_x11_get_type (), NULL);
 
-  return default_keymap;
+  display_x11->keymap->display = display;
+
+  return display_x11->keymap;
 }
 
 /* Find the index of the group/level pair within the keysyms for a key.
  */
-#define KEYSYM_INDEX(group, level) \
-  (2 * ((group) % (keysyms_per_keycode / 2)) + (level))
+#define KEYSYM_INDEX(keymap_impl, group, level) \
+  (2 * ((group) % (keymap_impl->keysyms_per_keycode / 2)) + (level))
+#define KEYSYM_IS_KEYPAD(s) (((s) >= 0xff80 && (s) <= 0xffbd) || \
+                             ((s) >= 0x11000000 && (s) <= 0x1100ffff))
 
 static void
-update_keymaps (void)
+update_keymaps (GdkKeymapX11 *keymap_x11)
 {
-  static guint current_serial = 0;
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (GDK_KEYMAP (keymap_x11)->display);
+  Display *xdisplay = display_x11->xdisplay;
   
 #ifdef HAVE_XKB
-  g_assert (!_gdk_use_xkb);
+  g_assert (!KEYMAP_USE_XKB (GDK_KEYMAP (keymap_x11)));
 #endif
   
-  if (keymap == NULL ||
-      current_serial != _gdk_keymap_serial)
+  if (keymap_x11->keymap == NULL ||
+      keymap_x11->current_serial != display_x11->keymap_serial)
     {
       gint i;
       gint map_size;
       gint keycode;
-      
-      current_serial = _gdk_keymap_serial;
 
-      update_keyrange ();
+      keymap_x11->current_serial = display_x11->keymap_serial;
       
-      if (keymap)
-        XFree (keymap);
+      update_keyrange (keymap_x11);
+      
+      if (keymap_x11->keymap)
+        XFree (keymap_x11->keymap);
 
-      if (mod_keymap)
-        XFreeModifiermap (mod_keymap);
+      if (keymap_x11->mod_keymap)
+        XFreeModifiermap (keymap_x11->mod_keymap);
       
-      keymap = XGetKeyboardMapping (gdk_display, min_keycode,
-                                    max_keycode - min_keycode,
-                                    &keysyms_per_keycode);
+      keymap_x11->keymap = XGetKeyboardMapping (xdisplay, keymap_x11->min_keycode,
+						keymap_x11->max_keycode - keymap_x11->min_keycode + 1,
+						&keymap_x11->keysyms_per_keycode);
 
 
       /* GDK_ISO_Left_Tab, as usually configured through XKB, really messes
@@ -163,31 +249,31 @@ update_keymaps (void)
        * However, <shift>Tab is not usually GDK_ISO_Left_Tab without XKB,
        * we we fudge the map here.
        */
-      keycode = min_keycode;
-      while (keycode < max_keycode)
+      keycode = keymap_x11->min_keycode;
+      while (keycode <= keymap_x11->max_keycode)
         {
-          KeySym *syms = keymap + (keycode - min_keycode) * keysyms_per_keycode;
+          KeySym *syms = keymap_x11->keymap + (keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
 	  /* Check both groups */
 	  for (i = 0 ; i < 2 ; i++)
 	    {
-	      if (syms[KEYSYM_INDEX (i, 0)] == GDK_Tab)
-		syms[KEYSYM_INDEX (i, 1)] = GDK_ISO_Left_Tab;
+	      if (syms[KEYSYM_INDEX (keymap_x11, i, 0)] == GDK_Tab)
+		syms[KEYSYM_INDEX (keymap_x11, i, 1)] = GDK_ISO_Left_Tab;
 	    }
 
           /*
            * If there is one keysym and the key symbol has upper and lower
            * case variants fudge the keymap
            */
-          if (syms[KEYSYM_INDEX (0, 1)] == 0)
+          if (syms[KEYSYM_INDEX (keymap_x11, 0, 1)] == 0)
             {
               guint lower;
               guint upper;
 
-              gdk_keyval_convert_case (syms[KEYSYM_INDEX (0, 0)], &lower, &upper);
+              gdk_keyval_convert_case (syms[KEYSYM_INDEX (keymap_x11, 0, 0)], &lower, &upper);
               if (lower != upper)
                 {
-                  syms[KEYSYM_INDEX (0, 0)] = lower;
-                  syms[KEYSYM_INDEX (0, 1)] = upper;
+                  syms[KEYSYM_INDEX (keymap_x11, 0, 0)] = lower;
+                  syms[KEYSYM_INDEX (keymap_x11, 0, 1)] = upper;
                 }
             }
       
@@ -195,80 +281,125 @@ update_keymaps (void)
           ++keycode;
         }
 
-      mod_keymap = XGetModifierMapping (gdk_display);
+      keymap_x11->mod_keymap = XGetModifierMapping (xdisplay);
 
+      keymap_x11->lock_keysym = GDK_VoidSymbol;
+      keymap_x11->group_switch_mask = 0;
+      keymap_x11->num_lock_mask = 0;
 
-      group_switch_mask = 0;
-
-      /* there are 8 modifiers, and the first 3 are shift, shift lock,
-       * and control
+      /* There are 8 sets of modifiers, with each set containing
+       * max_keypermod keycodes.
        */
-      map_size = 8 * mod_keymap->max_keypermod;
-      i = 3 * mod_keymap->max_keypermod;
-      while (i < map_size)
+      map_size = 8 * keymap_x11->mod_keymap->max_keypermod;
+      for (i = 0; i < map_size; i++)
         {
-          /* get the key code at this point in the map,
-           * see if its keysym is GDK_Mode_switch, if so
-           * we have the mode key
+          /* Get the key code at this point in the map. */
+          gint keycode = keymap_x11->mod_keymap->modifiermap[i];
+          gint j;
+          KeySym *syms;
+	  guint mask;
+
+          /* Ignore invalid keycodes. */
+          if (keycode < keymap_x11->min_keycode ||
+              keycode > keymap_x11->max_keycode)
+            continue;
+
+          syms = keymap_x11->keymap + (keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
+
+          /* The fourth modifier, GDK_MOD1_MASK is 1 << 3.
+	   * Each group of max_keypermod entries refers to the same modifier.
            */
-          gint keycode = mod_keymap->modifiermap[i];
-      
-          if (keycode >= min_keycode &&
-              keycode <= max_keycode)
+          mask = 1 << (i / keymap_x11->mod_keymap->max_keypermod);
+
+          switch (mask)
             {
-              gint j = 0;
-              KeySym *syms = keymap + (keycode - min_keycode) * keysyms_per_keycode;
-              while (j < keysyms_per_keycode)
+            case GDK_LOCK_MASK:
+              /* Get the Lock keysym.  If any keysym bound to the Lock modifier
+               * is Caps_Lock, we will interpret the modifier as Caps_Lock;
+               * otherwise, if any is bound to Shift_Lock, we will interpret
+               * the modifier as Shift_Lock. Otherwise, the lock modifier
+	       * has no effect.
+               */
+	      for (j = 0; j < keymap_x11->keysyms_per_keycode; j++)
+		{
+		  if (syms[j] == GDK_Caps_Lock)
+		    keymap_x11->lock_keysym = GDK_Caps_Lock;
+		  else if (syms[j] == GDK_Shift_Lock &&
+			   keymap_x11->lock_keysym == GDK_VoidSymbol)
+		    keymap_x11->lock_keysym = GDK_Shift_Lock;
+		}
+              break;
+
+            case GDK_CONTROL_MASK:
+            case GDK_SHIFT_MASK:
+            case GDK_MOD1_MASK:
+              /* Some keyboard maps are known to map Mode_Switch as an
+               * extra Mod1 key. In circumstances like that, it won't be
+               * used to switch groups.
+               */
+              break;
+
+            default:
+              /* Find the Mode_Switch and Num_Lock modifiers. */
+              for (j = 0; j < keymap_x11->keysyms_per_keycode; j++)
                 {
                   if (syms[j] == GDK_Mode_switch)
                     {
                       /* This modifier swaps groups */
-
-                      /* GDK_MOD1_MASK is 1 << 3 for example, i.e. the
-                       * fourth modifier, i / keyspermod is the modifier
-                       * index
-                       */
-                  
-                      group_switch_mask |= (1 << ( i / mod_keymap->max_keypermod));
-                      break;
+                      keymap_x11->group_switch_mask |= mask;
+                    }
+                  else if (syms[j] == GDK_Num_Lock)
+                    {
+                      /* This modifier is used for Num_Lock */
+                      keymap_x11->num_lock_mask |= mask;
                     }
-              
-                  ++j;
                 }
+              break;
             }
-      
-          ++i;
         }
+
+      /* Hack: The Sun X server puts the keysym to use when the Num Lock
+       * modifier is on in the third element of the keysym array, instead
+       * of the second.
+       */
+      if ((strcmp (ServerVendor (xdisplay), "Sun Microsystems, Inc.") == 0) &&
+          (keymap_x11->keysyms_per_keycode > 2))
+        keymap_x11->sun_keypad = TRUE;
+      else
+        keymap_x11->sun_keypad = FALSE;
     }
 }
 
 static const KeySym*
-get_keymap (void)
+get_keymap (GdkKeymapX11 *keymap_x11)
 {
-  update_keymaps ();  
+  update_keymaps (keymap_x11);
   
-  return keymap;
+  return keymap_x11->keymap;
 }
 
 #if HAVE_XKB
 static PangoDirection
-get_direction (void)
+get_direction (GdkKeymapX11 *keymap_x11)
 {
-  XkbDescRec *xkb = get_xkb ();
+  XkbDescRec *xkb = get_xkb (keymap_x11);
   const char *name;
   XkbStateRec state_rec;
   PangoDirection result;
 
-  XkbGetState (gdk_display, XkbUseCoreKbd, &state_rec);
+  GdkDisplay *display = GDK_KEYMAP (keymap_x11)->display;
 
+  XkbGetState (GDK_DISPLAY_XDISPLAY (display), XkbUseCoreKbd, &state_rec);
+  
   if (xkb->names->groups[state_rec.locked_group] == None)
     result = PANGO_DIRECTION_LTR;
   else
     {
-      name = gdk_x11_get_xatom_name (xkb->names->groups[state_rec.locked_group]);
-      if (g_strcasecmp (name, "arabic") == 0 ||
-	  g_strcasecmp (name, "hebrew") == 0 ||
-	  g_strcasecmp (name, "israelian") == 0)
+      name = gdk_x11_get_xatom_name_for_display (display, xkb->names->groups[state_rec.locked_group]);
+
+      if (g_ascii_strcasecmp (name, "arabic") == 0 ||
+	  g_ascii_strcasecmp (name, "hebrew") == 0 ||
+	  g_ascii_strcasecmp (name, "israelian") == 0)
 	result = PANGO_DIRECTION_RTL;
       else
 	result = PANGO_DIRECTION_LTR;
@@ -278,35 +409,61 @@ get_direction (void)
 }
 
 void
-_gdk_keymap_state_changed (void)
+_gdk_keymap_state_changed (GdkDisplay *display)
 {
-  if (default_keymap)
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
+  
+  if (display_x11->keymap)
     {
-      PangoDirection new_direction = get_direction ();
+      GdkKeymapX11 *keymap_x11 = GDK_KEYMAP_X11 (display_x11->keymap);
       
-      if (!have_direction || new_direction != current_direction)
+      PangoDirection new_direction = get_direction (keymap_x11);
+      
+      if (!keymap_x11->have_direction || new_direction != keymap_x11->current_direction)
 	{
-	  have_direction = TRUE;
-	  current_direction = new_direction;
-	  g_signal_emit_by_name (G_OBJECT (default_keymap), "direction_changed");
+	  keymap_x11->have_direction = TRUE;
+	  keymap_x11->current_direction = new_direction;
+	  g_signal_emit_by_name (keymap_x11, "direction_changed");
 	}
     }
 }
+
 #endif /* HAVE_XKB */
+
+void
+_gdk_keymap_keys_changed (GdkDisplay *display)
+{
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
   
+  ++display_x11->keymap_serial;
+  
+  if (display_x11->keymap)
+    g_signal_emit_by_name (display_x11->keymap, "keys_changed", 0);
+}
+
 PangoDirection
 gdk_keymap_get_direction (GdkKeymap *keymap)
 {
-#if HAVE_XKB
-  if (_gdk_use_xkb)
+  if (!keymap)
     {
-      if (!have_direction)
+      keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+      GDK_NOTE (MULTIHEAD,
+		g_message ("_multihead : reverting to default display keymap "
+			   "in gdk_keymap_get_direction"));
+    }
+  
+#if HAVE_XKB
+  if (KEYMAP_USE_XKB (keymap))
+    {
+      GdkKeymapX11 *keymap_x11 = GDK_KEYMAP_X11 (keymap);
+      
+      if (!keymap_x11->have_direction)
 	{
-	  current_direction = get_direction ();
-	  have_direction = TRUE;
+	  keymap_x11->current_direction = get_direction (keymap_x11);
+	  keymap_x11->have_direction = TRUE;
 	}
   
-      return current_direction;
+      return keymap_x11->current_direction;
     }
   else
 #endif /* HAVE_XKB */
@@ -341,25 +498,36 @@ gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
                                    gint          *n_keys)
 {
   GArray *retval;
+  GdkKeymapX11 *keymap_x11;
 
   g_return_val_if_fail (keymap == NULL || GDK_IS_KEYMAP (keymap), FALSE);
   g_return_val_if_fail (keys != NULL, FALSE);
   g_return_val_if_fail (n_keys != NULL, FALSE);
   g_return_val_if_fail (keyval != 0, FALSE);
+
+  if (!keymap)
+    {
+      keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+      GDK_NOTE (MULTIHEAD,
+		g_message ("_multihead : reverting to default display keymap "
+			   "in gdk_keymap_get_entries_for_keyval\n"));
+    }
+
+  keymap_x11 = GDK_KEYMAP_X11 (keymap);
   
   retval = g_array_new (FALSE, FALSE, sizeof (GdkKeymapKey));
 
 #ifdef HAVE_XKB
-  if (_gdk_use_xkb)
+  if (KEYMAP_USE_XKB (keymap))
     {
       /* See sec 15.3.4 in XKB docs */
 
-      XkbDescRec *xkb = get_xkb ();
+      XkbDescRec *xkb = get_xkb (keymap_x11);
       gint keycode;
       
-      keycode = min_keycode;
+      keycode = keymap_x11->min_keycode;
 
-      while (keycode <= max_keycode)
+      while (keycode <= keymap_x11->max_keycode)
         {
           gint max_shift_levels = XkbKeyGroupsWidth (xkb, keycode); /* "key width" */
           gint group = 0;
@@ -390,7 +558,8 @@ gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
 
                   g_array_append_val (retval, key);
 
-                  g_assert (XkbKeySymEntry (xkb, keycode, level, group) == keyval);
+                  g_assert (XkbKeySymEntry (xkb, keycode, level, group) == 
+			    keyval);
                 }
 
               ++level;
@@ -410,16 +579,16 @@ gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
   else
 #endif
     {
-      const KeySym *map = get_keymap ();
+      const KeySym *map = get_keymap (keymap_x11);
       gint keycode;
       
-      keycode = min_keycode;
-      while (keycode < max_keycode)
+      keycode = keymap_x11->min_keycode;
+      while (keycode <= keymap_x11->max_keycode)
         {
-          const KeySym *syms = map + (keycode - min_keycode) * keysyms_per_keycode;
+          const KeySym *syms = map + (keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
           gint i = 0;
 
-          while (i < keysyms_per_keycode)
+          while (i < keymap_x11->keysyms_per_keycode)
             {
               if (syms[i] == keyval)
                 {
@@ -482,16 +651,28 @@ gdk_keymap_get_entries_for_keycode (GdkKeymap     *keymap,
                                     guint        **keyvals,
                                     gint          *n_entries)
 {
+  GdkKeymapX11 *keymap_x11;
+  
   GArray *key_array;
   GArray *keyval_array;
 
   g_return_val_if_fail (keymap == NULL || GDK_IS_KEYMAP (keymap), FALSE);
   g_return_val_if_fail (n_entries != NULL, FALSE);
 
-  update_keyrange ();
+  if (!keymap)
+    {
+      keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+      GDK_NOTE (MULTIHEAD,
+		g_message ("_multihead : reverting to default display keymap "
+			   "in gdk_keymap_get_entries_for_keycode\n"));
+    }
 
-  if (hardware_keycode < min_keycode ||
-      hardware_keycode > max_keycode)
+  keymap_x11 = GDK_KEYMAP_X11 (keymap);
+
+  update_keyrange (keymap_x11);
+
+  if (hardware_keycode < keymap_x11->min_keycode ||
+      hardware_keycode > keymap_x11->max_keycode)
     {
       if (keys)
         *keys = NULL;
@@ -513,11 +694,11 @@ gdk_keymap_get_entries_for_keycode (GdkKeymap     *keymap,
     keyval_array = NULL;
   
 #ifdef HAVE_XKB
-  if (_gdk_use_xkb)
+  if (KEYMAP_USE_XKB (keymap))
     {
       /* See sec 15.3.4 in XKB docs */
 
-      XkbDescRec *xkb = get_xkb ();
+      XkbDescRec *xkb = get_xkb (keymap_x11);
       gint max_shift_levels;
       gint group = 0;
       gint level = 0;
@@ -567,13 +748,13 @@ gdk_keymap_get_entries_for_keycode (GdkKeymap     *keymap,
   else
 #endif
     {
-      const KeySym *map = get_keymap ();
+      const KeySym *map = get_keymap (keymap_x11);
       const KeySym *syms;
       gint i = 0;
 
-      syms = map + (hardware_keycode - min_keycode) * keysyms_per_keycode;
+      syms = map + (hardware_keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
 
-      while (i < keysyms_per_keycode)
+      while (i < keymap_x11->keysyms_per_keycode)
         {
           if (key_array)
             {
@@ -646,30 +827,45 @@ guint
 gdk_keymap_lookup_key (GdkKeymap          *keymap,
                        const GdkKeymapKey *key)
 {
+  GdkKeymapX11 *keymap_x11;
+  
   g_return_val_if_fail (keymap == NULL || GDK_IS_KEYMAP (keymap), 0);
   g_return_val_if_fail (key != NULL, 0);
   g_return_val_if_fail (key->group < 4, 0);
   
-#ifdef HAVE_XKB
-  if (_gdk_use_xkb)
+  if (!keymap)
     {
-      XkbDescRec *xkb = get_xkb ();
+      keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+      GDK_NOTE (MULTIHEAD,
+		g_message ("_multihead : reverting to default display keymap "
+			   "in gdk_keymap_lookup_key\n"));
+    }
+
+  keymap_x11 = GDK_KEYMAP_X11 (keymap);
+  
+#ifdef HAVE_XKB
+  if (KEYMAP_USE_XKB (keymap))
+    {
+      XkbDescRec *xkb = get_xkb (keymap_x11);
       
       return XkbKeySymEntry (xkb, key->keycode, key->level, key->group);
     }
   else
 #endif
     {
-      const KeySym *map = get_keymap ();
-      const KeySym *syms = map + (key->keycode - min_keycode) * keysyms_per_keycode;
-      return syms [KEYSYM_INDEX (key->group, key->level)];
+      const KeySym *map = get_keymap (keymap_x11);
+      const KeySym *syms = map + (key->keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
+      return syms [KEYSYM_INDEX (keymap_x11, key->group, key->level)];
     }
 }
 
 #ifdef HAVE_XKB
-/* This is copied straight from XFree86 Xlib, because I needed to
- * add the group and level return. It's unchanged for ease of
- * diff against the Xlib sources; don't reformat it.
+/* This is copied straight from XFree86 Xlib, to:
+ *  - add the group and level return.
+ *  - change the interpretation of mods_rtrn as described
+ *    in the docs for gdk_keymap_translate_keyboard_state()
+ * It's unchanged for ease of diff against the Xlib sources; don't
+ * reformat it.
  */
 static Bool
 MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
@@ -723,29 +919,48 @@ MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
     if (type->map) { /* find the column (shift level) within the group */
         register int i;
         register XkbKTMapEntryPtr entry;
+	/* ---- Begin section modified for GDK  ---- */
+	int found = 0;
+	
         for (i=0,entry=type->map;i<type->map_count;i++,entry++) {
-            if ((entry->active)&&((mods&type->mods.mask)==entry->mods.mask)) {
+	    if (mods_rtrn) {
+		int bits = 0;
+		unsigned long tmp = entry->mods.mask;
+		while (tmp) {
+		    if ((tmp & 1) == 1)
+			bits++;
+		    tmp >>= 1;
+		}
+		/* We always add one-modifiers levels to mods_rtrn since
+		 * they can't wipe out bits in the state unless the
+		 * level would be triggered. But return other modifiers
+		 * 
+		 */
+		if (bits == 1 || (mods&type->mods.mask)==entry->mods.mask)
+		    *mods_rtrn |= entry->mods.mask;
+	    }
+	    
+            if (!found&&entry->active&&((mods&type->mods.mask)==entry->mods.mask)) {
                 col+= entry->level;
                 if (type->preserve)
                     preserve= type->preserve[i].mask;
 
-                /* ---- Begin stuff GDK adds to the original Xlib version ---- */
-                
                 if (level_rtrn)
                   *level_rtrn = entry->level;
                 
-                /* ---- End stuff GDK adds to the original Xlib version ---- */
-                
-                break;
+                found = 1;
             }
         }
+	/* ---- End section modified for GDK ---- */
     }
 
     if (keysym_rtrn!=NULL)
         *keysym_rtrn= syms[col];
     if (mods_rtrn) {
-        *mods_rtrn= type->mods.mask&(~preserve);
-
+	/* ---- Begin section modified for GDK  ---- */
+        *mods_rtrn &= ~preserve;
+	/* ---- End section modified for GDK ---- */
+	
         /* ---- Begin stuff GDK comments out of the original Xlib version ---- */
         /* This is commented out because xkb_info is a private struct */
 
@@ -776,6 +991,97 @@ MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
 }
 #endif /* HAVE_XKB */
 
+/* Translates from keycode/state to keysymbol using the traditional interpretation
+ * of the keyboard map. See section 12.7 of the Xlib reference manual
+ */
+static guint
+translate_keysym (GdkKeymapX11   *keymap_x11,
+		  guint           hardware_keycode,
+		  gint            group,
+		  GdkModifierType state,
+		  guint          *effective_group,
+		  guint          *effective_level)
+{
+  const KeySym *map = get_keymap (keymap_x11);
+  const KeySym *syms = map + (hardware_keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
+
+#define SYM(k,g,l) syms[KEYSYM_INDEX (k,g,l)]
+
+  GdkModifierType shift_modifiers;
+  gint shift_level;
+  guint tmp_keyval;
+  gint num_lock_index;
+
+  shift_modifiers = GDK_SHIFT_MASK;
+  if (keymap_x11->lock_keysym == GDK_Shift_Lock)
+    shift_modifiers |= GDK_LOCK_MASK;
+
+  /* Fall back to the first group if the passed in group is empty
+   */
+  if (!(SYM (keymap_x11, group, 0) || SYM (keymap_x11, group, 1)) &&
+      (SYM (keymap_x11, 0, 0) || SYM (keymap_x11, 0, 1)))
+    group = 0;
+
+  /* Hack: On Sun, the Num Lock modifier uses the third element in the
+   * keysym array, and Mode_Switch does not apply for a keypad key.
+   */
+  if (keymap_x11->sun_keypad)
+    {
+      num_lock_index = 2;
+      
+      if (group != 0)
+	{
+	  gint i;
+	  
+	  for (i = 0; i < keymap_x11->keysyms_per_keycode; i++)
+	    if (KEYSYM_IS_KEYPAD (SYM (keymap_x11, 0, i)))
+	      group = 0;
+	}
+    }
+  else
+    num_lock_index = 1;
+
+  if ((state & keymap_x11->num_lock_mask) &&
+      KEYSYM_IS_KEYPAD (SYM (keymap_x11, group, num_lock_index)))
+    {
+      /* Shift, Shift_Lock cancel Num_Lock
+       */
+      shift_level = (state & shift_modifiers) ? 0 : num_lock_index;
+      if (!SYM (keymap_x11, group, shift_level) && SYM (keymap_x11, group, 0))
+        shift_level = 0;
+
+       tmp_keyval = SYM (keymap_x11, group, shift_level);
+    }
+  else
+    {
+      /* Fall back to the the first level if no symbol for the level
+       * we were passed.
+       */
+      shift_level = (state & shift_modifiers) ? 1 : 0;
+      if (!SYM (keymap_x11, group, shift_level) && SYM (keymap_x11, group, 0))
+	shift_level = 0;
+  
+      tmp_keyval = SYM (keymap_x11, group, shift_level);
+      
+      if (keymap_x11->lock_keysym == GDK_Caps_Lock && (state & GDK_LOCK_MASK) != 0)
+	{
+	  guint upper = gdk_keyval_to_upper (tmp_keyval);
+	  if (upper != tmp_keyval)
+	    tmp_keyval = upper;
+	}
+    }
+
+  if (effective_group)
+    *effective_group = group;
+      
+  if (effective_level)
+    *effective_level = shift_level;
+
+  return tmp_keyval;
+  
+#undef SYM
+}
+
 /**
  * gdk_keymap_translate_keyboard_state:
  * @keymap: a #GdkKeymap, or %NULL to use the default
@@ -797,6 +1103,49 @@ MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
  * affected by the active keyboard group. The @level is derived from
  * @state. For convenience, #GdkEventKey already contains the translated
  * keyval, so this function isn't as useful as you might think.
+ *
+ * <note><para>
+ * @consumed_modifiers gives modifiers that should be masked out
+ * from @state when comparing this key press to a hot key. For
+ * instance, on a US keyboard, the <literal>plus</literal>
+ * symbol is shifted, so when comparing a key press to a
+ * <literal>&lt;Control&gt;plus</literal> accelerator &lt;Shift&gt; should
+ * be masked out.
+ * </para>
+ * <informalexample><programlisting>
+ * &sol;* We want to ignore irrelevant modifiers like ScrollLock *&sol;
+ * &num;define ALL_ACCELS_MASK (GDK_CONTROL_MASK | GDK_SHIFT_MASK | GDK_MOD1_MASK)
+ * gdk_keymap_translate_keyboard_state (keymap, event->hardware_keycode,
+ *                                      event->state, event->group,
+ *                                      &amp;keyval, NULL, NULL, &amp;consumed);
+ * if (keyval == GDK_PLUS &&
+ *     (event->state &amp; ~consumed &amp; ALL_ACCELS_MASK) == GDK_CONTROL_MASK)
+ *   &sol;* Control was pressed *&sol;
+ * </programlisting></informalexample>
+ * <para>
+ * An older interpretation @consumed_modifiers was that it contained
+ * all modifiers that might affect the translation of the key;
+ * this allowed accelerators to be stored with irrelevant consumed
+ * modifiers, by doing:</para>
+ * <informalexample><programlisting>
+ * &sol;* XXX Don't do this XXX *&sol;
+ * if (keyval == accel_keyval &&
+ *     (event->state &amp; ~consumed &amp; ALL_ACCELS_MASK) == (accel_mods &amp; ~consumed))
+ *   &sol;* Accelerator was pressed *&sol;
+ * </programlisting></informalexample>
+ * <para>
+ * However, this did not work if multi-modifier combinations were
+ * used in the keymap, since, for instance, <literal>&lt;Control&gt;</literal>
+ * would be masked out even if only <literal>&lt;Control&gt;&lt;Alt&gt;</literal>
+ * was used in the keymap. To support this usage as well as well as
+ * possible, all <emphasis>single modifier</emphasis> combinations
+ * that could affect the key for any combination of modifiers will
+ * be returned in @consumed_modifiers; multi-modifier combinations
+ * are returned only when actually found in @state. When you store
+ * accelerators, you should always store them with consumed modifiers
+ * removed. Store <literal>&lt;Control&gt;plus</literal>,
+ * not <literal>&lt;Control&gt;&lt;Shift&gt;plus</literal>,
+ * </para></note>
  * 
  * Return value: %TRUE if there was a keyval bound to the keycode/state/group
  **/
@@ -810,12 +1159,15 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
                                      gint            *level,
                                      GdkModifierType *consumed_modifiers)
 {
+  GdkKeymapX11 *keymap_x11;
   KeySym tmp_keyval = NoSymbol;
   guint tmp_modifiers;
 
   g_return_val_if_fail (keymap == NULL || GDK_IS_KEYMAP (keymap), FALSE);
   g_return_val_if_fail (group < 4, FALSE);
   
+  keymap_x11 = GDK_KEYMAP_X11 (keymap);
+
   if (keyval)
     *keyval = NoSymbol;
   if (effective_group)
@@ -825,16 +1177,16 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
   if (consumed_modifiers)
     *consumed_modifiers = 0;
 
-  update_keyrange ();
+  update_keyrange (keymap_x11);
   
-  if (hardware_keycode < min_keycode ||
-      hardware_keycode > max_keycode)
+  if (hardware_keycode < keymap_x11->min_keycode ||
+      hardware_keycode > keymap_x11->max_keycode)
     return FALSE;
   
 #ifdef HAVE_XKB
-  if (_gdk_use_xkb)
+  if (KEYMAP_USE_XKB (keymap))
     {
-      XkbDescRec *xkb = get_xkb ();
+      XkbDescRec *xkb = get_xkb (keymap_x11);
 
       /* replace bits 13 and 14 with the provided group */
       state &= ~(1 << 13 | 1 << 14);
@@ -859,68 +1211,36 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
   else
 #endif
     {
-      const KeySym *map = get_keymap ();
-      const KeySym *syms;
-      gint shift_level;
-      gboolean ignore_shift = FALSE;
-      gboolean ignore_group = FALSE;
+      GdkModifierType bit;
       
-      if ((state & GDK_SHIFT_MASK) &&
-          (state & GDK_LOCK_MASK))
-	shift_level = 0; /* shift disables shift lock */
-      else if ((state & GDK_SHIFT_MASK) ||
-               (state & GDK_LOCK_MASK))
-	shift_level = 1;
-      else
-	shift_level = 0;
+      tmp_modifiers = 0;
 
-      syms = map + (hardware_keycode - min_keycode) * keysyms_per_keycode;
-
-#define SYM(g,l) syms[KEYSYM_INDEX (g,l)]
-
-      /* Drop group and shift if there are no keysymbols on
-       * the specified key.
+      /* We see what modifiers matter by trying the translation with
+       * and without each possible modifier
        */
-      if (!SYM (group, shift_level) && SYM (group, 0))
+      for (bit = GDK_SHIFT_MASK; bit < GDK_BUTTON1_MASK; bit <<= 1)
 	{
-	  shift_level = 0;
-	  ignore_shift = TRUE;
+	  /* Handling of the group here is a bit funky; a traditional
+	   * X keyboard map can have more than two groups, but no way
+	   * of accessing the extra groups is defined. We allow a
+	   * caller to pass in any group to this function, but we 
+	   * only can represent switching between group 0 and 1 in
+	   * consumed modifiers.
+	   */
+	  if (translate_keysym (keymap_x11, hardware_keycode,
+				(bit == keymap_x11->group_switch_mask) ? 0 : group,
+				state & ~bit,
+				NULL, NULL) !=
+	      translate_keysym (keymap_x11, hardware_keycode,
+				(bit == keymap_x11->group_switch_mask) ? 1 : group,
+				state | bit,
+				NULL, NULL))
+	    tmp_modifiers |= bit;
 	}
-      if (!SYM (group, shift_level) && SYM (0, shift_level))
-	{
-	  group = 0;
-	  ignore_group = TRUE;
-	}
-      if (!SYM (group, shift_level) && SYM (0, 0))
-	{
-	  shift_level = 0;
-	  group = 0;
-	  ignore_group = TRUE;
-	  ignore_shift = TRUE;
-	}
-
-      /* See whether the group and shift level actually mattered
-       * to know what to put in consumed_modifiers
-       */
-      if (!SYM (group, 1) ||
-	  SYM (group, 0) == SYM (group, 1))
-	ignore_shift = TRUE;
-
-      if (!SYM (1, shift_level) ||
-	  SYM (0, shift_level) == SYM (1, shift_level))
-	ignore_group = TRUE;
-
-      tmp_keyval = SYM (group, shift_level);
-
-      tmp_modifiers = ignore_group ? 0 : group_switch_mask;
-      tmp_modifiers |= ignore_shift ? 0 : (GDK_SHIFT_MASK | GDK_LOCK_MASK);
-
-      if (effective_group)
-        *effective_group = group;
-
-      if (level)
-        *level = shift_level;
-#undef SYM	  
+      
+      tmp_keyval = translate_keysym (keymap_x11, hardware_keycode,
+				     group, state,
+				     level, effective_group);
     }
 
   if (consumed_modifiers)
@@ -991,17 +1311,21 @@ gdk_keyval_convert_case (guint symbol,
 #endif /* HAVE_XCONVERTCASE */
 
 gint
-_gdk_x11_get_group_for_state (GdkModifierType state)
+_gdk_x11_get_group_for_state (GdkDisplay      *display,
+			      GdkModifierType  state)
 {
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
+  
 #ifdef HAVE_XKB
-  if (_gdk_use_xkb)
+  if (display_x11->use_xkb)
     {
       return XkbGroupForCoreState (state);
     }
   else
 #endif
     {
-      update_keymaps ();
-      return (state & group_switch_mask) ? 1 : 0;
+      GdkKeymapX11 *keymap_impl = GDK_KEYMAP_X11 (gdk_keymap_get_for_display (display));
+      update_keymaps (keymap_impl);
+      return (state & keymap_impl->group_switch_mask) ? 1 : 0;
     }
 }
