@@ -89,64 +89,64 @@ gtk_file_chooser_class_init (gpointer g_iface)
   
   g_object_interface_install_property (g_iface,
 				       g_param_spec_enum ("action",
-							  _("Action"),
-							  _("The type of operation that the file selector is performing"),
+							  P_("Action"),
+							  P_("The type of operation that the file selector is performing"),
 							  GTK_TYPE_FILE_CHOOSER_ACTION,
 							  GTK_FILE_CHOOSER_ACTION_OPEN,
 							  G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_object ("file-system",
-							    _("File System"),
-							    _("File system object to use"),
+							    P_("File System"),
+							    P_("File system object to use"),
 							    GTK_TYPE_FILE_SYSTEM,
 							    G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_object ("filter",
-							    _("Filter"),
-							    _("The current filter for selecting which files are displayed"),
+							    P_("Filter"),
+							    P_("The current filter for selecting which files are displayed"),
 							    GTK_TYPE_FILE_FILTER,
 							    G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_boolean ("folder-mode",
-							     _("Folder Mode"),
-							     _("Whether to select folders rather than files"),
+							     P_("Folder Mode"),
+							     P_("Whether to select folders rather than files"),
 							     FALSE,
 							     G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_boolean ("local-only",
-							     _("Local Only"),
-							     _("Whether the selected file(s) should be limited to local file: URLs"),
+							     P_("Local Only"),
+							     P_("Whether the selected file(s) should be limited to local file: URLs"),
 							     TRUE,
 							     G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_object ("preview-widget",
-							    _("Preview widget"),
-							    _("Application supplied widget for custom previews."),
+							    P_("Preview widget"),
+							    P_("Application supplied widget for custom previews."),
 							    GTK_TYPE_WIDGET,
 							    G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_boolean ("preview-widget-active",
-							     _("Preview Widget Active"),
-							     _("Whether the application supplied widget for custom previews should be shown."),
+							     P_("Preview Widget Active"),
+							     P_("Whether the application supplied widget for custom previews should be shown."),
 							     TRUE,
 							     G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_object ("extra-widget",
-							    _("Extra widget"),
-							    _("Application supplied widget for extra options."),
+							    P_("Extra widget"),
+							    P_("Application supplied widget for extra options."),
 							    GTK_TYPE_WIDGET,
 							    G_PARAM_READWRITE));
   g_object_interface_install_property (g_iface,
 				       g_param_spec_boolean ("select-multiple",
-							     _("Select Multiple"),
-							     _("Whether to allow multiple files to be selected"),
+							     P_("Select Multiple"),
+							     P_("Whether to allow multiple files to be selected"),
 							     FALSE,
 							     G_PARAM_READWRITE));
   
   g_object_interface_install_property (g_iface,
 				       g_param_spec_boolean ("show-hidden",
-							     _("Show Hidden"),
-							     _("Whether the hidden files and folders should be displayed"),
+							     P_("Show Hidden"),
+							     P_("Whether the hidden files and folders should be displayed"),
 							     FALSE,
 							     G_PARAM_READWRITE));
 }
@@ -361,6 +361,9 @@ gtk_file_chooser_get_select_multiple (GtkFileChooser *chooser)
  * Gets the filename for the currently selected file in
  * the file selector. If multiple files are selected,
  * one of the filenames will be returned at random.
+ *
+ * If the file chooser is in folder mode, this function returns the selected
+ * folder.
  * 
  * Return value: The currently selected filename, or %NULL
  *  if no file is selected, or the selected file can't
@@ -629,6 +632,9 @@ gtk_file_chooser_set_current_name  (GtkFileChooser *chooser,
  * the file selector. If multiple files are selected,
  * one of the filenames will be returned at random.
  * 
+ * If the file chooser is in folder mode, this function returns the selected
+ * folder.
+ * 
  * Return value: The currently selected URI, or %NULL
  *  if no file is selected. Free with g_free()
  *
