@@ -76,26 +76,28 @@ static void gtk_frame_real_compute_child_allocation (GtkFrame      *frame,
 static GtkBinClass *parent_class = NULL;
 
 
-GtkType
+GType
 gtk_frame_get_type (void)
 {
-  static GtkType frame_type = 0;
+  static GType frame_type = 0;
 
   if (!frame_type)
     {
-      static const GtkTypeInfo frame_info =
+      static const GTypeInfo frame_info =
       {
-	"GtkFrame",
-	sizeof (GtkFrame),
 	sizeof (GtkFrameClass),
-	(GtkClassInitFunc) gtk_frame_class_init,
-	(GtkObjectInitFunc) gtk_frame_init,
-        /* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_frame_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkFrame),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_frame_init,
       };
 
-      frame_type = gtk_type_unique (gtk_bin_get_type (), &frame_info);
+      frame_type = g_type_register_static (GTK_TYPE_BIN, "GtkFrame",
+					   &frame_info, 0);
     }
 
   return frame_type;
@@ -105,16 +107,14 @@ static void
 gtk_frame_class_init (GtkFrameClass *class)
 {
   GObjectClass *gobject_class;
-  GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
   gobject_class = (GObjectClass*) class;
-  object_class = GTK_OBJECT_CLASS (class);
   widget_class = GTK_WIDGET_CLASS (class);
   container_class = GTK_CONTAINER_CLASS (class);
 
-  parent_class = gtk_type_class (gtk_bin_get_type ());
+  parent_class = g_type_class_peek_parent (class);
 
   gobject_class->set_property = gtk_frame_set_property;
   gobject_class->get_property = gtk_frame_get_property;
@@ -122,43 +122,43 @@ gtk_frame_class_init (GtkFrameClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_LABEL,
                                    g_param_spec_string ("label",
-                                                        _("Label"),
-                                                        _("Text of the frame's label."),
+                                                        P_("Label"),
+                                                        P_("Text of the frame's label"),
                                                         NULL,
                                                         G_PARAM_READABLE |
 							G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
 				   PROP_LABEL_XALIGN,
-				   g_param_spec_double ("label_xalign",
-                                                        _("Label xalign"),
-                                                        _("The horizontal alignment of the label."),
-                                                        0.0,
-                                                        1.0,
-                                                        0.5,
-                                                        G_PARAM_READABLE |
-                                                        G_PARAM_WRITABLE));
+				   g_param_spec_float ("label_xalign",
+						       P_("Label xalign"),
+						       P_("The horizontal alignment of the label"),
+						       0.0,
+						       1.0,
+						       0.5,
+						       G_PARAM_READABLE |
+						       G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
 				   PROP_LABEL_YALIGN,
-				   g_param_spec_double ("label_yalign",
-                                                        _("Label yalign"),
-                                                        _("The vertical alignment of the label."),
-                                                        0.0,
-                                                        1.0,
-                                                        0.5,
-                                                        G_PARAM_READABLE |
-                                                        G_PARAM_WRITABLE));
+				   g_param_spec_float ("label_yalign",
+						       P_("Label yalign"),
+						       P_("The vertical alignment of the label"),
+						       0.0,
+						       1.0,
+						       0.5,
+						       G_PARAM_READABLE |
+						       G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW,
                                    g_param_spec_enum ("shadow", NULL,
-                                                      _("Deprecated property, use shadow_type instead."),
+                                                      P_("Deprecated property, use shadow_type instead"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_IN,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW_TYPE,
                                    g_param_spec_enum ("shadow_type",
-                                                      _("Frame shadow"),
-                                                      _("Appearance of the frame border."),
+                                                      P_("Frame shadow"),
+                                                      P_("Appearance of the frame border"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_IN,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -166,8 +166,8 @@ gtk_frame_class_init (GtkFrameClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_LABEL_WIDGET,
                                    g_param_spec_object ("label_widget",
-                                                        _("Label widget"),
-                                                        _("A widget to display in place of the usual frame label."),
+                                                        P_("Label widget"),
+                                                        P_("A widget to display in place of the usual frame label"),
                                                         GTK_TYPE_WIDGET,
                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
   
@@ -206,12 +206,12 @@ gtk_frame_set_property (GObject         *object,
       gtk_frame_set_label (frame, g_value_get_string (value));
       break;
     case PROP_LABEL_XALIGN:
-      gtk_frame_set_label_align (frame, g_value_get_double (value), 
+      gtk_frame_set_label_align (frame, g_value_get_float (value), 
 				 frame->label_yalign);
       break;
     case PROP_LABEL_YALIGN:
       gtk_frame_set_label_align (frame, frame->label_xalign, 
-				 g_value_get_double (value));
+				 g_value_get_float (value));
       break;
     case PROP_SHADOW:
     case PROP_SHADOW_TYPE:
@@ -242,10 +242,10 @@ gtk_frame_get_property (GObject         *object,
       g_value_set_string (value, gtk_frame_get_label (frame));
       break;
     case PROP_LABEL_XALIGN:
-      g_value_set_double (value, frame->label_xalign);
+      g_value_set_float (value, frame->label_xalign);
       break;
     case PROP_LABEL_YALIGN:
-      g_value_set_double (value, frame->label_yalign);
+      g_value_set_float (value, frame->label_yalign);
       break;
     case PROP_SHADOW:
     case PROP_SHADOW_TYPE:
@@ -265,13 +265,7 @@ gtk_frame_get_property (GObject         *object,
 GtkWidget*
 gtk_frame_new (const gchar *label)
 {
-  GtkFrame *frame;
-
-  frame = gtk_type_new (gtk_frame_get_type ());
-
-  gtk_frame_set_label (frame, label);
-
-  return GTK_WIDGET (frame);
+  return g_object_new (GTK_TYPE_FRAME, "label", label, NULL);
 }
 
 static void
@@ -319,8 +313,6 @@ gtk_frame_set_label (GtkFrame *frame,
 
       gtk_frame_set_label_widget (frame, child);
     }
-
-  g_object_notify (G_OBJECT (frame), "label");
 }
 
 /**
@@ -334,7 +326,8 @@ gtk_frame_set_label (GtkFrame *frame,
  * 
  * Return value: the text in the label, or %NULL if there
  *               was no label widget or the lable widget was not
- *               a #GtkLabel. This value must be freed with g_free().
+ *               a #GtkLabel. This string is owned by GTK+ and
+ *               must not be modified or freed.
  **/
 G_CONST_RETURN gchar *
 gtk_frame_get_label (GtkFrame *frame)
@@ -387,7 +380,10 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
   if (GTK_WIDGET_VISIBLE (frame) && need_resize)
     gtk_widget_queue_resize (GTK_WIDGET (frame));
 
+  g_object_freeze_notify (G_OBJECT (frame));
   g_object_notify (G_OBJECT (frame), "label_widget");
+  g_object_notify (G_OBJECT (frame), "label");
+  g_object_thaw_notify (G_OBJECT (frame));
 }
 
 /**
@@ -417,6 +413,7 @@ gtk_frame_set_label_align (GtkFrame *frame,
   xalign = CLAMP (xalign, 0.0, 1.0);
   yalign = CLAMP (yalign, 0.0, 1.0);
 
+  g_object_freeze_notify (G_OBJECT (frame));
   if (xalign != frame->label_xalign)
     {
       frame->label_xalign = xalign;
@@ -429,6 +426,7 @@ gtk_frame_set_label_align (GtkFrame *frame,
       g_object_notify (G_OBJECT (frame), "label_yalign");
     }
 
+  g_object_thaw_notify (G_OBJECT (frame));
   gtk_widget_queue_resize (GTK_WIDGET (frame));
 }
 
@@ -467,7 +465,7 @@ gtk_frame_set_shadow_type (GtkFrame      *frame,
 
       if (GTK_WIDGET_DRAWABLE (frame))
 	{
-	  gtk_widget_queue_clear (GTK_WIDGET (frame));
+	  gtk_widget_queue_draw (GTK_WIDGET (frame));
 	}
       
       gtk_widget_queue_resize (GTK_WIDGET (frame));
@@ -613,7 +611,7 @@ gtk_frame_size_allocate (GtkWidget     *widget,
        new_allocation.y != frame->child_allocation.y ||
        new_allocation.width != frame->child_allocation.width ||
        new_allocation.height != frame->child_allocation.height))
-    gtk_widget_queue_clear (widget);
+    gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
   
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
     gtk_widget_size_allocate (bin->child, &new_allocation);
