@@ -155,8 +155,8 @@ gtk_table_class_init (GtkTableClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_N_ROWS,
                                    g_param_spec_uint ("n_rows",
-						     _("Rows"),
-						     _("The number of rows in the table"),
+						     P_("Rows"),
+						     P_("The number of rows in the table"),
 						     0,
 						     G_MAXUINT,
 						     0,
@@ -164,8 +164,8 @@ gtk_table_class_init (GtkTableClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_N_COLUMNS,
                                    g_param_spec_uint ("n_columns",
-						     _("Columns"),
-						     _("The number of columns in the table"),
+						     P_("Columns"),
+						     P_("The number of columns in the table"),
 						     0,
 						     G_MAXUINT,
 						     0,
@@ -173,8 +173,8 @@ gtk_table_class_init (GtkTableClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_ROW_SPACING,
                                    g_param_spec_uint ("row_spacing",
-						     _("Row spacing"),
-						     _("The amount of space between two consecutive rows"),
+						     P_("Row spacing"),
+						     P_("The amount of space between two consecutive rows"),
 						     0,
 						     G_MAXUINT,
 						     0,
@@ -182,8 +182,8 @@ gtk_table_class_init (GtkTableClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_COLUMN_SPACING,
                                    g_param_spec_uint ("column_spacing",
-						     _("Column spacing"),
-						     _("The amount of space between two consecutive columns"),
+						     P_("Column spacing"),
+						     P_("The amount of space between two consecutive columns"),
 						     0,
 						     G_MAXUINT,
 						     0,
@@ -191,65 +191,65 @@ gtk_table_class_init (GtkTableClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_HOMOGENEOUS,
                                    g_param_spec_boolean ("homogeneous",
-							 _("Homogenous"),
-							 _("If TRUE this means the table cells are all the same width/height"),
+							 P_("Homogenous"),
+							 P_("If TRUE this means the table cells are all the same width/height"),
 							 FALSE,
 							 G_PARAM_READWRITE));
 
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_LEFT_ATTACH,
 					      g_param_spec_uint ("left_attach", 
-								 _("Left attachment"), 
-								 _("The column number to attach the left side of the child to"),
+								 P_("Left attachment"), 
+								 P_("The column number to attach the left side of the child to"),
 								 0, 65535, 0,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_RIGHT_ATTACH,
 					      g_param_spec_uint ("right_attach", 
-								 _("Right attachment"), 
-								 _("The column number to attach the right side of a child widget to"),
+								 P_("Right attachment"), 
+								 P_("The column number to attach the right side of a child widget to"),
 								 1, 65535, 1,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_TOP_ATTACH,
 					      g_param_spec_uint ("top_attach", 
-								 _("Top attachment"), 
-								 _("The row number to attach the top of a child widget to"),
+								 P_("Top attachment"), 
+								 P_("The row number to attach the top of a child widget to"),
 								 0, 65535, 0,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_BOTTOM_ATTACH,
 					      g_param_spec_uint ("bottom_attach",
-								 _("Bottom attachment"), 
-								 _("The row number to attach the bottom of the child to"),
+								 P_("Bottom attachment"), 
+								 P_("The row number to attach the bottom of the child to"),
 								 1, 65535, 1,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_X_OPTIONS,
 					      g_param_spec_flags ("x_options", 
-								  _("Horizontal options"), 
-								  _("Options specifying the horizontal behaviour of the child"),
+								  P_("Horizontal options"), 
+								  P_("Options specifying the horizontal behaviour of the child"),
 								  GTK_TYPE_ATTACH_OPTIONS, GTK_EXPAND | GTK_FILL,
 								  G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_Y_OPTIONS,
 					      g_param_spec_flags ("y_options", 
-								  _("Vertical options"), 
-								  _("Options specifying the vertical behaviour of the child"),
+								  P_("Vertical options"), 
+								  P_("Options specifying the vertical behaviour of the child"),
 								  GTK_TYPE_ATTACH_OPTIONS, GTK_EXPAND | GTK_FILL,
 								  G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_X_PADDING,
 					      g_param_spec_uint ("x_padding", 
-								 _("Horizontal padding"), 
-								 _("Extra space to put between the child and its left and right neighbors, in pixels"),
+								 P_("Horizontal padding"), 
+								 P_("Extra space to put between the child and its left and right neighbors, in pixels"),
 								 0, 65535, 0,
 								 G_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_Y_PADDING,
 					      g_param_spec_uint ("y_padding", 
-								 _("Vertical padding"), 
-								 _("Extra space to put between the child and its upper and lower neighbors, in pixels"),
+								 P_("Vertical padding"), 
+								 P_("Extra space to put between the child and its upper and lower neighbors, in pixels"),
 								 0, 65535, 0,
 								 G_PARAM_READWRITE));
 }
